/****************************************************************************
** Meta object code from reading C++ file 'optionsDialog.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "optionsDialog.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'optionsDialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15XxOptionsDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto XxOptionsDialog::qt_create_metaobjectdata<qt_meta_tag_ZN15XxOptionsDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "XxOptionsDialog",
        "synchronize",
        "",
        "onApply",
        "onClose",
        "checkboxIgnoreTrailingBlanks",
        "checkboxIgnoreWhitespace",
        "checkboxIgnoreCase",
        "checkboxIgnoreBlankLines",
        "radiobuttonQualityNormal",
        "radiobuttonQualityFastest",
        "radiobuttonQualityHighest",
        "editFontApp",
        "editFontText",
        "listboxColors",
        "QListWidgetItem*",
        "editColorFore",
        "editColorBack"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'synchronize'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onApply'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onClose'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'checkboxIgnoreTrailingBlanks'
        QtMocHelpers::SlotData<void(int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'checkboxIgnoreWhitespace'
        QtMocHelpers::SlotData<void(int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'checkboxIgnoreCase'
        QtMocHelpers::SlotData<void(int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'checkboxIgnoreBlankLines'
        QtMocHelpers::SlotData<void(int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'radiobuttonQualityNormal'
        QtMocHelpers::SlotData<void(bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'radiobuttonQualityFastest'
        QtMocHelpers::SlotData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'radiobuttonQualityHighest'
        QtMocHelpers::SlotData<void(bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'editFontApp'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editFontText'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'listboxColors'
        QtMocHelpers::SlotData<void(QListWidgetItem *, QListWidgetItem *)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 2 }, { 0x80000000 | 15, 2 },
        }}),
        // Slot 'editColorFore'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editColorBack'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<XxOptionsDialog, qt_meta_tag_ZN15XxOptionsDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject XxOptionsDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15XxOptionsDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15XxOptionsDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15XxOptionsDialogE_t>.metaTypes,
    nullptr
} };

void XxOptionsDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<XxOptionsDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->synchronize(); break;
        case 1: _t->onApply(); break;
        case 2: _t->onClose(); break;
        case 3: _t->checkboxIgnoreTrailingBlanks((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->checkboxIgnoreWhitespace((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->checkboxIgnoreCase((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->checkboxIgnoreBlankLines((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->radiobuttonQualityNormal((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->radiobuttonQualityFastest((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->radiobuttonQualityHighest((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->editFontApp(); break;
        case 11: _t->editFontText(); break;
        case 12: _t->listboxColors((*reinterpret_cast<std::add_pointer_t<QListWidgetItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QListWidgetItem*>>(_a[2]))); break;
        case 13: _t->editColorFore(); break;
        case 14: _t->editColorBack(); break;
        default: ;
        }
    }
}

const QMetaObject *XxOptionsDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *XxOptionsDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15XxOptionsDialogE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::XxOptionsDialogBase"))
        return static_cast< Ui::XxOptionsDialogBase*>(this);
    return QDialog::qt_metacast(_clname);
}

int XxOptionsDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 15;
    }
    return _id;
}
QT_WARNING_POP
