/****************************************************************************
** Meta object code from reading C++ file 'app.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "app.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'app.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN5XxAppE_t {};
} // unnamed namespace

template <> constexpr inline auto XxApp::qt_create_metaobjectdata<qt_meta_tag_ZN5XxAppE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "XxApp",
        "cursorChanged",
        "",
        "cursorLine",
        "textSizeChanged",
        "updateWidgets",
        "updateWidgetsForDiffs",
        "onNbLinesChanged",
        "openLeft",
        "openMiddle",
        "openRight",
        "saveAsLeft",
        "saveAsMiddle",
        "saveAsRight",
        "saveAsMerged",
        "saveAs",
        "generatePatchFromLeft",
        "generatePatchFromMiddle",
        "generatePatchFromRight",
        "saveSelectedOnly",
        "editLeft",
        "editMiddle",
        "editRight",
        "saveOptions",
        "quit",
        "search",
        "searchForward",
        "searchBackward",
        "redoDiff",
        "editDiffOptions",
        "editDisplayOptions",
        "diffFilesAtCursor",
        "nextAndDiffFiles",
        "copyFileLeftToRight",
        "copyFileRightToLeft",
        "removeFileLeft",
        "removeFileRight",
        "nextDifference",
        "previousDifference",
        "nextUnselected",
        "previousUnselected",
        "selectGlobalLeft",
        "selectGlobalMiddle",
        "selectGlobalRight",
        "selectGlobalNeither",
        "selectGlobalUnselect",
        "selectGlobalUnselectedLeft",
        "selectGlobalUnselectedMiddle",
        "selectGlobalUnselectedRight",
        "selectGlobalUnselectedNeither",
        "selectGlobalMerge",
        "selectRegionLeft",
        "selectRegionMiddle",
        "selectRegionRight",
        "selectRegionNeither",
        "selectRegionUnselect",
        "selectRegionLeftAndNext",
        "selectRegionMiddleAndNext",
        "selectRegionRightAndNext",
        "selectRegionNeitherAndNext",
        "regionSplitSwapJoin",
        "selectLineLeft",
        "selectLineMiddle",
        "selectLineRight",
        "selectLineNeither",
        "selectLineUnselect",
        "tabsAt3",
        "tabsAt4",
        "tabsAt8",
        "ignoreTrailing",
        "ignoreWhitespace",
        "ignoreCase",
        "ignoreBlankLines",
        "hideCarriageReturns",
        "dirDiffRecursive",
        "ignoreFileChanges",
        "qualityNormal",
        "qualityFastest",
        "qualityHighest",
        "toggleShowFilenames",
        "togglePaneMergedView",
        "togglePopupMergedView",
        "toggleToolbar",
        "toggleLineNumbers",
        "toggleVerticalLine",
        "toggleHorizNullMarkers",
        "toggleOverview",
        "hordiffTypeNone",
        "hordiffTypeSingle",
        "hordiffTypeMultiple",
        "hordiffTypeUpdate",
        "toggleIgnoreHorizontalWs",
        "toggleIgnorePerHunkWs",
        "toggleFormatClipboardText",
        "ignoreFileNone",
        "ignoreFileLeft",
        "ignoreFileMiddle",
        "ignoreFileRight",
        "helpManPage",
        "helpAbout",
        "pageDown",
        "pageUp",
        "cursorDown",
        "cursorUp",
        "cursorTop",
        "cursorBottom",
        "quitAccept",
        "quitReject",
        "quitMerged",
        "fontSizeIncrease",
        "fontSizeDecrease"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'cursorChanged'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'textSizeChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateWidgets'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateWidgetsForDiffs'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onNbLinesChanged'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'openLeft'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'openMiddle'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'openRight'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'saveAsLeft'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'saveAsMiddle'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'saveAsRight'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'saveAsMerged'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'saveAs'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'generatePatchFromLeft'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'generatePatchFromMiddle'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'generatePatchFromRight'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'saveSelectedOnly'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editLeft'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editMiddle'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editRight'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'saveOptions'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'quit'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'search'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'searchForward'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'searchBackward'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'redoDiff'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editDiffOptions'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editDisplayOptions'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'diffFilesAtCursor'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'nextAndDiffFiles'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'copyFileLeftToRight'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'copyFileRightToLeft'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'removeFileLeft'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'removeFileRight'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'nextDifference'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'previousDifference'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'nextUnselected'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'previousUnselected'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectGlobalLeft'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectGlobalMiddle'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectGlobalRight'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectGlobalNeither'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectGlobalUnselect'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectGlobalUnselectedLeft'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectGlobalUnselectedMiddle'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectGlobalUnselectedRight'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectGlobalUnselectedNeither'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectGlobalMerge'
        QtMocHelpers::SlotData<int()>(50, 2, QMC::AccessPublic, QMetaType::Int),
        // Slot 'selectRegionLeft'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectRegionMiddle'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectRegionRight'
        QtMocHelpers::SlotData<void()>(53, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectRegionNeither'
        QtMocHelpers::SlotData<void()>(54, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectRegionUnselect'
        QtMocHelpers::SlotData<void()>(55, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectRegionLeftAndNext'
        QtMocHelpers::SlotData<void()>(56, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectRegionMiddleAndNext'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectRegionRightAndNext'
        QtMocHelpers::SlotData<void()>(58, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectRegionNeitherAndNext'
        QtMocHelpers::SlotData<void()>(59, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'regionSplitSwapJoin'
        QtMocHelpers::SlotData<void()>(60, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectLineLeft'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectLineMiddle'
        QtMocHelpers::SlotData<void()>(62, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectLineRight'
        QtMocHelpers::SlotData<void()>(63, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectLineNeither'
        QtMocHelpers::SlotData<void()>(64, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectLineUnselect'
        QtMocHelpers::SlotData<void()>(65, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'tabsAt3'
        QtMocHelpers::SlotData<void()>(66, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'tabsAt4'
        QtMocHelpers::SlotData<void()>(67, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'tabsAt8'
        QtMocHelpers::SlotData<void()>(68, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ignoreTrailing'
        QtMocHelpers::SlotData<void()>(69, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ignoreWhitespace'
        QtMocHelpers::SlotData<void()>(70, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ignoreCase'
        QtMocHelpers::SlotData<void()>(71, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ignoreBlankLines'
        QtMocHelpers::SlotData<void()>(72, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'hideCarriageReturns'
        QtMocHelpers::SlotData<void()>(73, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'dirDiffRecursive'
        QtMocHelpers::SlotData<void()>(74, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ignoreFileChanges'
        QtMocHelpers::SlotData<void()>(75, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'qualityNormal'
        QtMocHelpers::SlotData<void()>(76, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'qualityFastest'
        QtMocHelpers::SlotData<void()>(77, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'qualityHighest'
        QtMocHelpers::SlotData<void()>(78, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'toggleShowFilenames'
        QtMocHelpers::SlotData<void()>(79, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'togglePaneMergedView'
        QtMocHelpers::SlotData<void()>(80, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'togglePopupMergedView'
        QtMocHelpers::SlotData<void()>(81, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'toggleToolbar'
        QtMocHelpers::SlotData<void()>(82, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'toggleLineNumbers'
        QtMocHelpers::SlotData<void()>(83, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'toggleVerticalLine'
        QtMocHelpers::SlotData<void()>(84, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'toggleHorizNullMarkers'
        QtMocHelpers::SlotData<void()>(85, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'toggleOverview'
        QtMocHelpers::SlotData<void()>(86, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'hordiffTypeNone'
        QtMocHelpers::SlotData<void()>(87, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'hordiffTypeSingle'
        QtMocHelpers::SlotData<void()>(88, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'hordiffTypeMultiple'
        QtMocHelpers::SlotData<void()>(89, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'hordiffTypeUpdate'
        QtMocHelpers::SlotData<void()>(90, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'toggleIgnoreHorizontalWs'
        QtMocHelpers::SlotData<void()>(91, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'toggleIgnorePerHunkWs'
        QtMocHelpers::SlotData<void()>(92, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'toggleFormatClipboardText'
        QtMocHelpers::SlotData<void()>(93, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ignoreFileNone'
        QtMocHelpers::SlotData<void()>(94, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ignoreFileLeft'
        QtMocHelpers::SlotData<void()>(95, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ignoreFileMiddle'
        QtMocHelpers::SlotData<void()>(96, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'ignoreFileRight'
        QtMocHelpers::SlotData<void()>(97, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'helpManPage'
        QtMocHelpers::SlotData<void()>(98, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'helpAbout'
        QtMocHelpers::SlotData<void()>(99, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'pageDown'
        QtMocHelpers::SlotData<void()>(100, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'pageUp'
        QtMocHelpers::SlotData<void()>(101, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'cursorDown'
        QtMocHelpers::SlotData<void()>(102, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'cursorUp'
        QtMocHelpers::SlotData<void()>(103, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'cursorTop'
        QtMocHelpers::SlotData<void()>(104, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'cursorBottom'
        QtMocHelpers::SlotData<void()>(105, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'quitAccept'
        QtMocHelpers::SlotData<void()>(106, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'quitReject'
        QtMocHelpers::SlotData<void()>(107, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'quitMerged'
        QtMocHelpers::SlotData<void()>(108, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fontSizeIncrease'
        QtMocHelpers::SlotData<void()>(109, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fontSizeDecrease'
        QtMocHelpers::SlotData<void()>(110, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<XxApp, qt_meta_tag_ZN5XxAppE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject XxApp::staticMetaObject = { {
    QMetaObject::SuperData::link<QkApplication::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5XxAppE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5XxAppE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN5XxAppE_t>.metaTypes,
    nullptr
} };

void XxApp::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<XxApp *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->cursorChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->textSizeChanged(); break;
        case 2: _t->updateWidgets(); break;
        case 3: _t->updateWidgetsForDiffs(); break;
        case 4: _t->onNbLinesChanged(); break;
        case 5: _t->openLeft(); break;
        case 6: _t->openMiddle(); break;
        case 7: _t->openRight(); break;
        case 8: _t->saveAsLeft(); break;
        case 9: _t->saveAsMiddle(); break;
        case 10: _t->saveAsRight(); break;
        case 11: _t->saveAsMerged(); break;
        case 12: _t->saveAs(); break;
        case 13: _t->generatePatchFromLeft(); break;
        case 14: _t->generatePatchFromMiddle(); break;
        case 15: _t->generatePatchFromRight(); break;
        case 16: _t->saveSelectedOnly(); break;
        case 17: _t->editLeft(); break;
        case 18: _t->editMiddle(); break;
        case 19: _t->editRight(); break;
        case 20: _t->saveOptions(); break;
        case 21: _t->quit(); break;
        case 22: _t->search(); break;
        case 23: _t->searchForward(); break;
        case 24: _t->searchBackward(); break;
        case 25: _t->redoDiff(); break;
        case 26: _t->editDiffOptions(); break;
        case 27: _t->editDisplayOptions(); break;
        case 28: _t->diffFilesAtCursor(); break;
        case 29: _t->nextAndDiffFiles(); break;
        case 30: _t->copyFileLeftToRight(); break;
        case 31: _t->copyFileRightToLeft(); break;
        case 32: _t->removeFileLeft(); break;
        case 33: _t->removeFileRight(); break;
        case 34: _t->nextDifference(); break;
        case 35: _t->previousDifference(); break;
        case 36: _t->nextUnselected(); break;
        case 37: _t->previousUnselected(); break;
        case 38: _t->selectGlobalLeft(); break;
        case 39: _t->selectGlobalMiddle(); break;
        case 40: _t->selectGlobalRight(); break;
        case 41: _t->selectGlobalNeither(); break;
        case 42: _t->selectGlobalUnselect(); break;
        case 43: _t->selectGlobalUnselectedLeft(); break;
        case 44: _t->selectGlobalUnselectedMiddle(); break;
        case 45: _t->selectGlobalUnselectedRight(); break;
        case 46: _t->selectGlobalUnselectedNeither(); break;
        case 47: { int _r = _t->selectGlobalMerge();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 48: _t->selectRegionLeft(); break;
        case 49: _t->selectRegionMiddle(); break;
        case 50: _t->selectRegionRight(); break;
        case 51: _t->selectRegionNeither(); break;
        case 52: _t->selectRegionUnselect(); break;
        case 53: _t->selectRegionLeftAndNext(); break;
        case 54: _t->selectRegionMiddleAndNext(); break;
        case 55: _t->selectRegionRightAndNext(); break;
        case 56: _t->selectRegionNeitherAndNext(); break;
        case 57: _t->regionSplitSwapJoin(); break;
        case 58: _t->selectLineLeft(); break;
        case 59: _t->selectLineMiddle(); break;
        case 60: _t->selectLineRight(); break;
        case 61: _t->selectLineNeither(); break;
        case 62: _t->selectLineUnselect(); break;
        case 63: _t->tabsAt3(); break;
        case 64: _t->tabsAt4(); break;
        case 65: _t->tabsAt8(); break;
        case 66: _t->ignoreTrailing(); break;
        case 67: _t->ignoreWhitespace(); break;
        case 68: _t->ignoreCase(); break;
        case 69: _t->ignoreBlankLines(); break;
        case 70: _t->hideCarriageReturns(); break;
        case 71: _t->dirDiffRecursive(); break;
        case 72: _t->ignoreFileChanges(); break;
        case 73: _t->qualityNormal(); break;
        case 74: _t->qualityFastest(); break;
        case 75: _t->qualityHighest(); break;
        case 76: _t->toggleShowFilenames(); break;
        case 77: _t->togglePaneMergedView(); break;
        case 78: _t->togglePopupMergedView(); break;
        case 79: _t->toggleToolbar(); break;
        case 80: _t->toggleLineNumbers(); break;
        case 81: _t->toggleVerticalLine(); break;
        case 82: _t->toggleHorizNullMarkers(); break;
        case 83: _t->toggleOverview(); break;
        case 84: _t->hordiffTypeNone(); break;
        case 85: _t->hordiffTypeSingle(); break;
        case 86: _t->hordiffTypeMultiple(); break;
        case 87: _t->hordiffTypeUpdate(); break;
        case 88: _t->toggleIgnoreHorizontalWs(); break;
        case 89: _t->toggleIgnorePerHunkWs(); break;
        case 90: _t->toggleFormatClipboardText(); break;
        case 91: _t->ignoreFileNone(); break;
        case 92: _t->ignoreFileLeft(); break;
        case 93: _t->ignoreFileMiddle(); break;
        case 94: _t->ignoreFileRight(); break;
        case 95: _t->helpManPage(); break;
        case 96: _t->helpAbout(); break;
        case 97: _t->pageDown(); break;
        case 98: _t->pageUp(); break;
        case 99: _t->cursorDown(); break;
        case 100: _t->cursorUp(); break;
        case 101: _t->cursorTop(); break;
        case 102: _t->cursorBottom(); break;
        case 103: _t->quitAccept(); break;
        case 104: _t->quitReject(); break;
        case 105: _t->quitMerged(); break;
        case 106: _t->fontSizeIncrease(); break;
        case 107: _t->fontSizeDecrease(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (XxApp::*)(int )>(_a, &XxApp::cursorChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (XxApp::*)()>(_a, &XxApp::textSizeChanged, 1))
            return;
    }
}

const QMetaObject *XxApp::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *XxApp::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5XxAppE_t>.strings))
        return static_cast<void*>(this);
    return QkApplication::qt_metacast(_clname);
}

int XxApp::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QkApplication::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 108)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 108;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 108)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 108;
    }
    return _id;
}

// SIGNAL 0
void XxApp::cursorChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void XxApp::textSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
